from random import random, gauss, choice
from math import sin, cos, pi, log, sqrt, exp
from sys import stderr

# unused debugging decorator
def rlog(f):
    def _(*args):
        r = f(*args)
        print >> stderr, "returning %r from %s(%r)" % (r, f.func_name, args)
    return _


class circle:
	"""
	quadruple of (x,y, radius, color)
	printable as SVG circle
	with a shift method to offset the x,y
	"""
    def __init__(self, x, y, r, col="black"):
        self.x = x
        self.y = y
        self.r = r
        self.col = col

    def __str__(self):
        return ('<circle cx="%d" cy="%d" r="%f" fill="none" stroke="%s" ' +
                'stroke-width="0.4" />') % (self.x, self.y, self.r, self.col)

    def shift(self, xy):
        x, y = xy
        return circle(self.x+x, self.y+y, self.r, self.col)

def bubblify(f, sz, var):
    """
    takes a function f: [0,360] -> R x R
          a function sz: [0,360], R -> R
          a variance var: R
    returns a function that takes N and returns N circles along the graph of f,
          with gaussian offsets of variance var and radii as computed by sz
          when given the input for f and the offset
    """
    def _(n):
        cs = []
        cd = cdf(f, 0, 360, 0.5)
        for i in range(n):
            p = cd[choice(cd.keys())]   # random point on curve
            x1, y1 = f(p - 0.5)
            x2, y2 = f(p + 0.5)
            v = (-y2 + y1, x2 - x1)      # vector perpendicular to the curve
            vl = sqrt(v[0]**2 + v[1]**2)
            v = map(lambda p: p/vl, v)  # normed vector
            o = gauss(0, var)           # random offset
            rrr = sz(p, o)              # radius computed by sz
            v = v[0] * o, v[1] * o
            cs.append((((x1 + x2)/2, (y1 + y2)/2), v, rrr, p, o))
        return cs
    return _


# given the output of bubblify's return function,
# cast them to circles (as above)
def to_circles(cs):
    cicls = []
    for c in cs:
        xy, v, rrr, p, o = c
        cicls.append(circle(xy[0] + v[0], xy[1] + v[1], rrr))
    return cicls

# unused, SVG-presents the random offset vectors on the graph
def to_lines(cs):
    ln = []
    for i, c in enumerate(cs):
        xy, v, rrr, p, o = c
        ln.append('<line x1="%d" y1="%d" x2="%d" y2="%d"' +
                  ' stroke="black"/>' % (xy[0], xy[1], xy[0]+v[0], xy[1]+v[1]))
    return ln

# like numpy.arange
def space(a, b, s):
    n = int((b - a)/(s * 1.0))
    return [x*s+a for x in xrange(n)]

def cdf(f, a, b, eps):
    """
    returns a dict of equidistant floats between 0.0 and 1.0 of
    points on the graph of f between (a,f(a)) and (b,f(b))
    """
    def pathlen():
        """
        primitive path integral
        """
        l = 0
        h = {}
        p = f(a)
        h[a] = f(a)

        def dst(xy, vw):
            return sqrt((xy[0]-vw[0])**2 + (xy[1] - vw[1])**2)

        for x in space(a, b, eps)[1:]:
            pt = f(x)
            h[x] = dst(p, pt)
            l += h[x]
            p = pt[:]
        return l, h
    total, h = pathlen()
    cd = {a: 0}
    xs = sorted(h.keys())
    for x, y in zip(xs[1:], xs[:-1]):
        cd[x] = cd[y] + h[x]

    def reverse(d):
        return dict(zip(d.values(), d.keys()))

    def interpolate(cd, x):
        a = max(filter(lambda y: y <= x, cd.keys()))
        b = min(filter(lambda y: y > x, cd.keys()))
        return cd[a] + (x - a)/(b-a)*(cd[a]-cd[b])
    return {x: interpolate(reverse(cd), x) for x
            in space(0, total, eps*(b-a)/total)}

def svgbubble(cs):
    """
    Given a list of objects having .x, .y., .r accessors and a shift method
	(e.g. the circle class above), generate
    an SVG document with appropriate bounding box
    containing all the objects stringified 
    """
    xs = map(lambda e: e.x, cs)
    ys = map(lambda e: e.y, cs)
    rs = map(lambda e: e.r, cs)
        
    preamble = """
    <svg height="%d"
         width="%d"
         version="1.1"
         xmlns="http://www.w3.org/2000/svg"
         xmlns:xlink="http://www.w3.org/1999/xlink">
    """ % (max(ys) - min(ys)+max(rs), max(xs) - min(xs)+max(rs))
    postamble = "</svg>"
    
    cs = map(lambda c: c.shift((- min(xs), -min(ys))), cs)
    return preamble +"\n" + "\n".join( map(str,cs)) +"\n" +postamble


def main():
    r = 100  # scale factor
    # a function from [0, 360] -> R^2
    g = lambda x: r * sin(2*(x-180)/180*pi) * (1 - abs(x-180)/180)**2 *\
        sqrt(abs(1 - abs(x-180) / 180))
    
    # function to compute the radii depending on the input to g and 
    # the offset from g's graph
    # will be used as param sz to bubblify()
    def radii(p, o):
		# inversly logarithmic in the offset, linearily smaller towards the boundaries
        return (r/20) * 1 / (log(abs(o)+1)+1) * (1-abs((p-180)/220))

    b = bubblify(lambda x: (x, g(x)), radii, r/16)
    a = to_circles(b(400))
    print(svgbubble(a))

main()
